@@ -50,8 +50,8 @@ data StructType =
     | Tuple [StructType]
     | OneOf [StructType]
     | Any                               -- ^ XML's contentspec allows ANY
-    | StringMixed			-- ^ mixed (#PCDATA | ... )*
-    | String				-- ^ string only (#PCDATA)
+    | StringMixed                       -- ^ mixed (#PCDATA | ... )*
+    | String                            -- ^ string only (#PCDATA)
     | Defined Name
     deriving Eq
 
@@ -69,9 +69,9 @@ instance Show StructType where
                                     . foldr1 (.) (intersperse (showChar '|')
                                                               (map shows ss))
                                     . showChar ')'
-    showsPrec _ (Any)             = showString "ANY"
-    showsPrec _ (StringMixed)     = showString "#PCDATA"
-    showsPrec _ (String)          = showString "#PCDATA"
+    showsPrec _ Any               = showString "ANY"
+    showsPrec _ StringMixed       = showString "#PCDATA"
+    showsPrec _ String            = showString "#PCDATA"
     showsPrec _ (Defined (Name n _)) = showString n
 
 
