@@ -26,7 +26,7 @@ import Text.XML.HaXml.Posn
 import Text.ParserCombinators.Poly.Plain
 
 --  #define DEBUG
- 
+
 #if defined(DEBUG)
 #  if ( defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ > 502 ) || \
       ( defined(__NHC__) && __NHC__ > 114 ) || defined(__HUGS__)
@@ -54,7 +54,7 @@ htmlParse file = Prelude.either error id . htmlParse' file
 --   contents of the file.  The result is the generic representation of
 --   an XML document.  Any parsing errors are returned in the @Either@ type.
 htmlParse' :: String -> String -> Either String (Document Posn)
-htmlParse' file = Prelude.either Left (Right . simplify) . fst
+htmlParse' file = fmap simplify . fst
                   . runParser document . xmlLex file
 
 ---- Document simplification ----
@@ -115,7 +115,7 @@ closes :: Name -> Name -> Bool
 "form"  `closes` "form"   = True
 "label" `closes` "label"  = True
 _       `closes` "option" = True
-"thead" `closes` t        | t `elem` ["colgroup"]          = True
+"thead" `closes` t        | t == "colgroup"                = True
 "tfoot" `closes` t        | t `elem` ["thead","colgroup"]  = True
 "tbody" `closes` t        | t `elem` ["tbody","tfoot","thead","colgroup"] = True
 "colgroup" `closes` "colgroup"  = True
@@ -141,8 +141,8 @@ qname = fmap N name
 name :: HParser Name
 --name = do {(p,TokName s) <- next; return s}
 name = do (p,tok) <- next
-          case tok of 
-            TokName s  -> return s 
+          case tok of
+            TokName s  -> return s
             TokError _ -> report failBad "a name" p tok
             _          -> report fail "a name" p tok
 
@@ -156,13 +156,13 @@ freetext = do (p,t) <- next
 
 maybe :: HParser a -> HParser (Maybe a)
 maybe p =
-    ( p >>= return . Just) `onFail`
-    ( return Nothing)
+    (Just <$> p) `onFail`
+    return Nothing
 
 either :: HParser a -> HParser b -> HParser (Either a b)
 either p q =
-    ( p >>= return . Left) `onFail`
-    ( q >>= return . Right)
+    (Left <$> p) `onFail`
+    (Right <$> q)
 
 word :: String -> HParser ()
 word s = do { x <- next
@@ -180,7 +180,7 @@ posn = do { x@(p,_) <- next
           } `onFail` return noPos
 
 nmtoken :: HParser NmToken
-nmtoken = (string `onFail` freetext)
+nmtoken = string `onFail` freetext
 
 failP, failBadP :: String -> HParser a
 failP msg    = do { p <- posn; fail (msg++"\n    at "++show p) }
@@ -236,7 +236,7 @@ prolog = do
 xmldecl :: HParser XMLDecl
 xmldecl = do
     tok TokPIOpen
-    (word "xml" `onFail` word "XML")
+    word "xml" `onFail` word "XML"
     p <- posn
     s <- freetext
     tok TokPIClose `onFail` failBadP "missing ?> in <?xml ...?>"
@@ -250,14 +250,14 @@ xmldecl = do
 
 versioninfo :: HParser VersionInfo
 versioninfo = do
-    (word "version" `onFail` word "VERSION")
+    word "version" `onFail` word "VERSION"
     tok TokEqual
     bracket (tok TokQuote) (commit $ tok TokQuote) freetext
 
 misc :: HParser Misc
-misc = 
-    oneOf' [ ("<!--comment-->", comment >>= return . Comment)
-           , ("<?PI?>",         processinginstruction >>= return . PI)
+misc =
+    oneOf' [ ("<!--comment-->", Comment <$> comment)
+           , ("<?PI?>",         PI <$> processinginstruction)
            ]
 
 
@@ -278,11 +278,11 @@ doctypedecl = do
 
 --markupdecl :: HParser MarkupDecl
 --markupdecl =
---    ( elementdecl >>= return . Element) `onFail`
---    ( attlistdecl >>= return . AttList) `onFail`
---    ( entitydecl >>= return . Entity) `onFail`
---    ( notationdecl >>= return . Notation) `onFail`
---    ( misc >>= return . MarkupMisc) `onFail`
+--    (Element <$> elementdecl) `onFail`
+--    (AttList <$> attlistdecl) `onFail`
+--    (Entity <$> entitydecl) `onFail`
+--    (Notation <$> notationdecl) `onFail`
+--    (MarkupMisc <$> misc) `onFail`
 --    PEREF(MarkupPE,markupdecl)
 --
 --extsubset :: HParser ExtSubset
@@ -293,13 +293,13 @@ doctypedecl = do
 --
 --extsubsetdecl :: HParser ExtSubsetDecl
 --extsubsetdecl =
---    ( markupdecl >>= return . ExtMarkupDecl) `onFail`
---    ( conditionalsect >>= return . ExtConditionalSect) `onFail`
+--    (ExtMarkupDecl <$> markupdecl) `onFail`
+--    (ExtConditionalSect <$> conditionalsect) `onFail`
 --    PEREF(ExtPEReference,extsubsetdecl)
 
 sddecl :: HParser SDDecl
 sddecl = do
-    (word "standalone" `onFail` word "STANDALONE")
+    word "standalone" `onFail` word "STANDALONE"
     commit $ do
       tok TokEqual `onFail` failP "missing = in 'standalone' decl"
       bracket (tok TokQuote) (commit $ tok TokQuote)
@@ -323,27 +323,27 @@ element ctx =
     (ElemTag (N e) avs) <- elemtag
     ( if e `closes` ctx then
          -- insert the missing close-tag, fail forward, and reparse.
-         ( do debug ("/")
+         ( do debug "/"
               unparse ([TokEndOpen, TokName ctx, TokAnyClose,
                         TokAnyOpen, TokName e] ++ reformatAttrs avs)
               return ([], Elem (N "null") [] []))
       else if e `elem` selfclosingtags then
          -- complete the parse straightaway.
-         ( do tok TokEndClose	-- self-closing <tag /> 
+         ( do tok TokEndClose   -- self-closing <tag />
               debug (e++"[+]")
               return ([], Elem (N e) avs [])) `onFail`
-     --  ( do tok TokAnyClose	-- sequence <tag></tag>	(**not HTML?**)
+     --  ( do tok TokAnyClose   -- sequence <tag></tag> (**not HTML?**)
      --       debug (e++"[+")
      --       n <- bracket (tok TokEndOpen) (commit $ tok TokAnyClose) qname
      --       debug "]"
-     --       if e == (map toLower n :: Name) 
-     --         then return ([], Elem e avs [])      
+     --       if e == (map toLower n :: Name)
+     --         then return ([], Elem e avs [])
      --         else return (error "no nesting in empty tag")) `onFail`
-         ( do tok TokAnyClose	-- <tag> with no close (e.g. <IMG>)
+         ( do tok TokAnyClose   -- <tag> with no close (e.g. <IMG>)
               debug (e++"[+]")
               return ([], Elem (N e) avs []))
       else
-        (( do tok TokEndClose
+         ( do tok TokEndClose
               debug (e++"[]")
               return ([], Elem (N e) avs [])) `onFail`
          ( do tok TokAnyClose `onFail` failP "missing > or /> in element tag"
@@ -364,8 +364,8 @@ element ctx =
                 else
                   do unparse [TokEndOpen, TokName n, TokAnyClose]
                      debug "-"
-                     return (((e,avs):s), Elem (N e) avs cs))
-         ) `onFail` failP ("failed to repair non-matching tags in context: "++ctx)))
+                     return ((e,avs):s, Elem (N e) avs cs))
+         ) `onFail` failP ("failed to repair non-matching tags in context: "++ctx))
 
 closeInner :: Name -> [(Name,[Attribute])] -> [(Name,[Attribute])]
 closeInner c ts =
@@ -378,12 +378,12 @@ unparse ts = do p <- posn
                 reparse (zip (repeat p) ts)
 
 reformatAttrs :: [(QName, AttValue)] -> [TokenT]
-reformatAttrs avs = concatMap f0 avs
+reformatAttrs = concatMap f0
     where f0 (a, v@(AttValue _)) = [ TokName (printableName a), TokEqual
                                    , TokQuote, TokFreeText (show v), TokQuote ]
 
 reformatTags :: [(String, [(QName, AttValue)])] -> [TokenT]
-reformatTags ts = concatMap f0 ts
+reformatTags = concatMap f0
     where f0 (t,avs) = [TokAnyOpen, TokName t]++reformatAttrs avs++[TokAnyClose]
 
 content :: Name -> HParser (Stack,Content Posn)
@@ -408,7 +408,7 @@ attribute = do
     (N n) <- qname
     v <- (do tok TokEqual
              attvalue) `onFail`
-         (return (AttValue [Left "TRUE"]))
+         return (AttValue [Left "TRUE"])
     return (N $ map toLower n, v)
 
 --elementdecl :: HParser ElementDecl
@@ -424,8 +424,8 @@ attribute = do
 --contentspec =
 --    ( word "EMPTY" >> return EMPTY) `onFail`
 --    ( word "ANY" >> return ANY) `onFail`
---    ( mixed >>= return . Mixed) `onFail`
---    ( cp >>= return . ContentSpec) `onFail`
+--    (Mixed <$> mixed) `onFail`
+--    (ContentSpec <$> cp) `onFail`
 --    PEREF(ContentPE,contentspec)
 --
 --choice :: HParser [CP]
@@ -492,8 +492,8 @@ attribute = do
 --atttype :: HParser AttType
 --atttype =
 --    ( word "CDATA" >> return StringType) `onFail`
---    ( tokenizedtype >>= return . TokenizedType) `onFail`
---    ( enumeratedtype >>= return . EnumeratedType)
+--    (TokenizedType <$> tokenizedtype) `onFail`
+--    (EnumeratedType <$> enumeratedtype)
 --
 --tokenizedtype :: HParser TokenizedType
 --tokenizedtype =
@@ -507,8 +507,8 @@ attribute = do
 --
 --enumeratedtype :: HParser EnumeratedType
 --enumeratedtype =
---    ( notationtype >>= return . NotationType) `onFail`
---    ( enumeration >>= return . Enumeration)
+--    (NotationType <$> notationtype) `onFail`
+--    (Enumeration <$> enumeration)
 --
 --notationtype :: HParser NotationType
 --notationtype = do
@@ -555,7 +555,7 @@ attribute = do
 --    return (IgnoreSectContents i is)
 --
 --ignore :: HParser Ignore
---ignore = freetext >>= return . Ignore
+--ignore = Ignore <$> freetext
 
 reference :: HParser Reference
 reference = do
@@ -570,8 +570,8 @@ reference = do
 {-
 reference :: HParser Reference
 reference =
-    ( charref >>= return . RefChar) `onFail`
-    ( entityref >>= return . RefEntity)
+    (RefChar <$> charref) `onFail`
+    (RefEntity <$> entityref)
 
 entityref :: HParser EntityRef
 entityref = do
@@ -593,8 +593,8 @@ charref = do
 --
 --entitydecl :: HParser EntityDecl
 --entitydecl =
---    ( gedecl >>= return . EntityGEDecl) `onFail`
---    ( pedecl >>= return . EntityPEDecl)
+--    (EntityGEDecl <$> gedecl) `onFail`
+--    (EntityPEDecl <$> pedecl)
 --
 --gedecl :: HParser GEDecl
 --gedecl = do
@@ -617,25 +617,24 @@ charref = do
 --
 --entitydef :: HParser EntityDef
 --entitydef =
---    ( entityvalue >>= return . DefEntityValue) `onFail`
+--    (DefEntityValue <$> entityvalue) `onFail`
 --    ( do eid <- externalid
 --         ndd <- maybe ndatadecl
 --         return (DefExternalID eid ndd))
 --
 --pedef :: HParser PEDef
 --pedef =
---    ( entityvalue >>= return . PEDefEntityValue) `onFail`
---    ( externalid >>= return . PEDefExternalID)
+--    (PEDefEntityValue <$> entityvalue) `onFail`
+--    (PEDefExternalID <$> externalid)
 
 externalid :: HParser ExternalID
 externalid =
     ( do word "SYSTEM"
-         s <- systemliteral
-         return (SYSTEM s)) `onFail`
+         SYSTEM <$> systemliteral) `onFail`
     ( do word "PUBLIC"
          p <- pubidliteral
-         s <- (systemliteral `onFail` return (SystemLiteral ""))
-         return (PUBLIC p s))
+         PUBLIC p <$> systemliteral `onFail` return (SystemLiteral "")
+    )
 
 --ndatadecl :: HParser NDataDecl
 --ndatadecl = do
@@ -666,7 +665,7 @@ externalid =
 
 encodingdecl :: HParser EncodingDecl
 encodingdecl = do
-    (word "encoding" `onFail` word "ENCODING")
+    word "encoding" `onFail` word "ENCODING"
     tok TokEqual `onFail` failBadP "expected = in 'encoding' decl"
     f <- bracket (tok TokQuote) (commit $ tok TokQuote) freetext
     return (EncodingDecl f)
@@ -693,9 +692,9 @@ encodingdecl = do
 
 --ev :: HParser EV
 --ev =
---    ( freetext >>= return . EVString) `onFail`
+--    (EVString <$> freetext) `onFail`
 -- -- PEREF(EVPERef,ev) `onFail`
---    ( reference >>= return . EVRef)
+--    (EVRef <$> reference)
 
 attvalue :: HParser AttValue
 attvalue =
@@ -715,13 +714,12 @@ attvalue =
 systemliteral :: HParser SystemLiteral
 systemliteral = do
     s <- bracket (tok TokQuote) (commit $ tok TokQuote) freetext
-    return (SystemLiteral s)		-- note: need to fold &...; escapes
+    return (SystemLiteral s)            -- note: need to fold &...; escapes
 
 pubidliteral :: HParser PubidLiteral
 pubidliteral = do
     s <- bracket (tok TokQuote) (commit $ tok TokQuote) freetext
-    return (PubidLiteral s)		-- note: need to fold &...; escapes
+    return (PubidLiteral s)             -- note: need to fold &...; escapes
 
 chardata :: HParser CharData
-chardata = freetext -- >>= return . CharData
-
+chardata = freetext -- <&> CharData
