@@ -1,13 +1,13 @@
 module Text.XML.HaXml.TypeMapping
   (
   -- * A class to get an explicit type representation for any value
-    HTypeable(..)	-- sole method, toHType
+    HTypeable(..)       -- sole method, toHType
   -- * Explicit representation of Haskell datatype information
-  , HType(..)		-- instance of Eq, Show
-  , Constr(..)		-- instance of Eq, Show
+  , HType(..)           -- instance of Eq, Show
+  , Constr(..)          -- instance of Eq, Show
   -- * Helper functions to extract type info as strings
-  , showHType		-- :: HType -> ShowS
-  , showConstr		-- :: Int -> HType -> String
+  , showHType           -- :: HType -> ShowS
+  , showConstr          -- :: Int -> HType -> String
   -- * Conversion from Haskell datatype to DTD
   , toDTD
   ) where
@@ -38,9 +38,9 @@ data HType =
     | Prim String String        -- ^ separate Haskell name and XML name
     | String
     | Defined String [HType] [Constr]
-	-- ^ A user-defined type has a name, a sequence of type variables,
-	--   and a set of constructors.  (The variables might already be
-	--   instantiated to actual types.)
+        -- ^ A user-defined type has a name, a sequence of type variables,
+        --   and a set of constructors.  (The variables might already be
+        --   instantiated to actual types.)
     deriving (Show)
 
 instance Eq HType where
@@ -49,7 +49,7 @@ instance Eq HType where
     (Tuple xs) == (Tuple ys) =  xs==ys
     (Prim x _) == (Prim y _) =  x==y
     String     == String     =  True
-    (Defined n _xs _) == (Defined m _ys _)  =  n==m 	-- && xs==ys
+    (Defined n _xs _) == (Defined m _ys _)  =  n==m     -- && xs==ys
     _          == _          =  False
 
 -- | A concrete representation of any user-defined Haskell constructor.
@@ -202,11 +202,11 @@ toDTD ht =
   DTD (toplevel ht) Nothing (macrosFirst (reverse (h2d True [] [] [ht])))
   where
     macrosFirst :: [MarkupDecl] -> [MarkupDecl]
-    macrosFirst decls = concat [p, p'] where (p, p') = partition f decls
-                                             f (Entity _) = True
-                                             f _ = False
+    macrosFirst decls = p ++ p' where (p, p') = partition f decls
+                                      f (Entity _) = True
+                                      f _ = False
     toplevel ht@(Defined _ _ _) = N $ showHType ht "-XML"
-    toplevel ht@_               = N $ showHType ht ""
+    toplevel ht                 = N $ showHType ht ""
     c0 = False
     h2d :: Bool -> [HType] -> [Constr] -> [HType] -> [MarkupDecl]
     -- toplevel?   history    history   remainingwork     result
@@ -238,7 +238,7 @@ toDTD ht =
     declConstr chist c@(Constr s fv hts)
       | c `notElem` chist = [Element (ElementDecl (N $ flatConstr c "")
                                          (ContentSpec (constrHtExpr c)))]
-      | otherwise = [] 
+      | otherwise = []
     declprim (Prim _ t) =
       [ Element (ElementDecl (N t) EMPTY)
       , AttList (AttListDecl (N t) [AttDef (N "value") StringType REQUIRED])]
@@ -261,13 +261,13 @@ showHType (Tuple hts) = showString "tuple" . shows (length hts)
 showHType (Prim _ t)  = showString t
 showHType String      = showString "string"
 showHType (Defined s fv _)
-                      = showString s . ((length fv > 0) ? (showChar '-'))
+                      = showString s . ((length fv > 0) ? showChar '-')
                         . foldr (.) id (intersperse (showChar '-')
                                                     (map showHType fv))
 
 flatConstr :: Constr -> ShowS
 flatConstr (Constr s fv _)
-        = showString s . ((length fv > 0) ? (showChar '-'))
+        = showString s . ((length fv > 0) ? showChar '-')
           . foldr (.) id (intersperse (showChar '-') (map showHType fv))
 
 outerHtExpr :: HType -> CP
@@ -279,7 +279,7 @@ outerHtExpr ht              = innerHtExpr ht None
 
 innerHtExpr :: HType -> Modifier -> CP
 innerHtExpr (Prim _ t)  m = TagName (N t) m
-innerHtExpr (Tuple hts) m = Seq (map (\c-> innerHtExpr c None) hts) m
+innerHtExpr (Tuple hts) m = Seq (map (`innerHtExpr` None) hts) m
 innerHtExpr ht@(Defined _ _ _) m = -- CPPE (showHType ht "") (outerHtExpr ht)
                                    TagName (N ('%': showHType ht ";")) m
                                                         --  ***HACK!!!***
